import { Controller } from '@hotwired/stimulus';
import * as pdfjsLib from 'pdfjs-dist';
import { renderThumbnails } from 'helpers/pdf_thumbnails';

export default class extends Controller {
  static targets = [
    'content',
    'indexIndicator',
    'totalPagesIndicator',
    'zoomIndicator',
    'nextPageButton',
    'previousPageButton',
    'zoomInButton',
    'zoomOutButton',
    'thumbnailContainer',
  ];

  static values = {
    src: String,
    index: { type: Number, default: 1 },
    zoom: { type: Number, default: 1 },
    totalPages: { type: Number, default: 1 },
    active: { type: String, default: 'border-spirit-red hover:border-spirit-red' },
    workerUrl: String,
  };

  static MAX_ZOOM = 3;
  static MIN_ZOOM = 0.5;
  static ZOOM_STEP = 0.25;
  static THUMBNAIL_SIZE = 150;

  connect() {
    this.loadPDF();
  }

  async loadPDF() {
    pdfjsLib.GlobalWorkerOptions.workerSrc = this.workerUrlValue;

    try {
      this.pdf = await pdfjsLib.getDocument({ url: this.srcValue, maxCanvasBytes: 10000000, disableFontFace: true, scale: 0.1 }).promise;
      this.totalPagesValue = this.pdf.numPages;
      this.totalPagesIndicatorTarget.textContent = this.totalPagesValue;
      this.renderCurrentPage();
      renderThumbnails.call(this, 'click->pdf-viewer#goToPage');
    } catch (error) {
      console.error('Error loading PDF:', error);
    }
  }

  async renderCurrentPage() {
    try {
      this.updateActiveThumbnail();
      const page = await this.pdf.getPage(this.indexValue);
      const viewport = page.getViewport({ scale: this.zoomValue });

      const canvas = this.contentTarget;
      const context = canvas.getContext('2d');

      canvas.height = viewport.height;
      canvas.width = viewport.width;

      const renderContext = {
        canvasContext: context,
        viewport: viewport,
      };

      await page.render(renderContext).promise;
      this.updateControls();
    } catch (error) {
      console.error('Error rendering page:', error);
    }
  }

  updateControls() {
    if (this.hasIndexIndicatorTarget) {
      this.indexIndicatorTarget.value = this.indexValue;
    }
    if (this.hasZoomIndicatorTarget) {
      this.zoomIndicatorTarget.textContent = Math.round(this.zoomValue * 100) + '%';
    }
    this.disableButtonIfNeeded(this.previousPageButtonTarget, this.indexValue <= 1);
    this.disableButtonIfNeeded(this.nextPageButtonTarget, this.indexValue >= this.totalPagesValue);
  }

  nextPage() {
    if (this.indexValue < this.totalPagesValue) {
      this.indexValue++;
      this.renderCurrentPage();
    }
  }

  previousPage() {
    if (this.indexValue > 1) {
      this.indexValue--;
      this.renderCurrentPage();
    }
  }

  updateActiveThumbnail() {
    this.thumbnailContainerTarget.querySelectorAll('.thumbnail').forEach((thumbnail, i) => {
      if (i + 1 === this.indexValue) {
        thumbnail.classList.add(...this.activeValue.split(' '));
      } else {
        thumbnail.classList.remove(...this.activeValue.split(' '));
      }
    });
  }

  zoomIn() {
    if (!this.atMaxZoom()) {
      this.zoomValue += this.constructor.ZOOM_STEP;
      this.renderCurrentPage();
    }
  }

  zoomOut() {
    if (!this.atMinZoom()) {
      this.zoomValue -= this.constructor.ZOOM_STEP;
      this.renderCurrentPage();
    }
  }

  atMaxZoom() {
    return this.zoomValue >= this.constructor.MAX_ZOOM;
  }

  atMinZoom() {
    return this.zoomValue <= this.constructor.MIN_ZOOM;
  }

  disableButtonIfNeeded(btn, condition) {
    if (btn) {
      btn.disabled = condition;
    }
  }

  indexIndicatorKeyDown(event) {
    if (event.key === 'Enter') {
      const newPage = parseInt(event.target.value);
      if (newPage && newPage >= 1 && newPage <= this.totalPagesValue) {
        this.indexValue = newPage;
        this.renderCurrentPage();
      } else {
        event.target.value = this.indexValue;
      }
    }
  }

  goToPage(event) {
    const page = parseInt(event.currentTarget.dataset.page);
    this.indexValue = page;
    this.renderCurrentPage();
  }
}
